Circle = require("libraries/classic"):extend()

local sqrt, pow = math.sqrt, math.pow

function Circle.get_distance(x1, y1, x2, y2)
    return sqrt(pow(x2 - x1, 2) + pow(y2 - y1, 2))
end

function Circle:new(x, y, radius)
    self.x = x
    self.y = y
    self.radius = radius
end

function Circle:move_to(x, y)
    self.x, self.y = x, y
end

function Circle:is_point_inside(x, y)
    return Circle.get_distance(self.x, self.y, x, y) <= self.radius
end

function Circle:is_overlapping(other)
    return Circle.get_distance(self.x, self.y, other.x, other.y) < self.radius + other.radius
end

function Circle:get_overlap(other)
    local distance = Circle.get_distance(self.x, self.y, other.x, other.y)
    local overlapping_length = math.max(0, self.radius + other.radius - distance)
    local dx, dy = other.x - self.x, other.y - self.y
    local overlap_x, overlap_y = 0, 0

    if distance > 0 then
        overlap_x = overlapping_length * dx / distance
        overlap_y = overlapping_length * dy / distance
    end

    return overlap_x, overlap_y
end

function Circle:draw_fill()
    love.graphics.circle("fill",self.x,self.y,self.radius,segments or self.radius)
end

function Circle:draw_line(segments)
    love.graphics.circle("line",self.x,self.y,self.radius,segments or self.radius)
end