local async = {}

local active_jobs = {}

function async.wait(seconds)
    if type(seconds) ~= "number" then error("[seconds] should be a number.") end
    coroutine.yield(seconds)
end

function async.wait_condition(condition_callback, ...)
    if type(condition_callback) ~= "function" then error("[condition_callback] should be a function.") end
    coroutine.yield(condition_callback, ...)
end

function async.update(delta_time)
    delta_time = delta_time or love.timer.getDelta()

    for i = #active_jobs, 1, -1 do
        local job = active_jobs[i]

        if coroutine.status(job._coroutine) == "suspended" then
            local resume_job = false

            if type(job.wait_info) == "number" then
                job.wait_info = job.wait_info - delta_time

                resume_job = job.wait_info <= 0
            elseif type(job.wait_info) == "function" then
                resume_job = job.wait_info(unpack(job.condition_arguments))
            else
                resume_job = true
            end 

            if resume_job then
                local resume_info = {coroutine.resume(job._coroutine)}
                local ok = table.remove(resume_info, 1) -- pop the status
            
                if not ok then 
                    error(table.remove(resume_info, 1)) -- pop the error
                end

                job.wait_info = table.remove(resume_info, 1) -- pop wait info
                job.condition_arguments = resume_info -- resume_info is now the array of arguments
            end
        elseif coroutine.status(job._coroutine) == "dead" then
            table.remove(active_jobs, i)
        end
    end
end

function async.clear()
    for i = #active_jobs, 1, -1 do
        table.remove(active_jobs, i)
    end
end

function async.new_job(coroutine_function)
    if type(coroutine_function) ~= "function" then error("[coroutine_function] should be a function.") end
    local job = {
        _coroutine = coroutine.create(coroutine_function),
        wait_info = nil,
        condition_arguments = {},
    }
    table.insert(active_jobs, job)
end

-- camelCase aliases
async.newJob = async.new_job
async.waitCondition = async.wait_condition

return async