local check = require(PATH..'.check')
local canvas = {}

local _width = love.graphics.getWidth()
local _height = love.graphics.getHeight()
local _base_canvas = love.graphics.newCanvas(love.graphics.getDimensions())
local _stored_canvases = {}

function canvas.set_size(width, height)
    check(width, 'width', 'number', 'strictly_positive integer')
    check(height, 'height', 'number', 'strictly_positive integer')

    _width = width
    _height = height

    _base_canvas = love.graphics.newCanvas(width, height)
end

function canvas.store_to_buffer(index)
    canvas.enable_buffer(index, 0,0,0,0)
    canvas.draw()
    canvas.disable()
end

function canvas.save(filename, index)
    local target = _base_canvas
    if index then
        target = canvas.get_buffer_canvas(index)
    end
    target:newImageData():encode('png', filename)
end

function canvas.load(filename, index)
    if index then
        canvas.enable_buffer(index)
    else
        canvas.enable()
    end
    love.graphics.setColor(1,1,1)
    love.graphics.draw(love.graphics.newImage(filename))
    canvas.disable()
end

function canvas.create_buffer(index)
    local buffer_canvas = love.graphics.newCanvas(_width, _height)
    _stored_canvases[index] = buffer_canvas
    return buffer_canvas
end

function canvas.delete(index)
    _stored_canvases[index] = nil
end

function canvas.get_base_canvas()
    return _base_canvas
end

function canvas.get_buffer_canvas(index)
    return _stored_canvases[index]
end

function canvas.enable(...)
    love.graphics.push()
    love.graphics.origin()
    love.graphics.setBlendMode('alpha','alphamultiply')
    love.graphics.setCanvas(_base_canvas)
    if select(1, ...) ~= nil then
        love.graphics.clear(...)
    end
end

function canvas.enable_buffer(index, ...)
    local buffer = canvas.get_buffer_canvas(index) or canvas.create_buffer(index)
    love.graphics.push()
    love.graphics.origin()
    love.graphics.setBlendMode('alpha','alphamultiply')
    love.graphics.setCanvas(buffer)
    if select(1, ...) ~= nil then
        love.graphics.clear(...)
    end
end

function canvas.disable()
    love.graphics.setCanvas()
    love.graphics.pop()
end

function canvas.draw(...)
    love.graphics.setBlendMode('alpha','premultiplied')
    if select(1, ...) then
        love.graphics.setColor(...)
    else
        love.graphics.setColor(1,1,1)
    end
    love.graphics.draw(_base_canvas)
    love.graphics.setBlendMode('alpha','alphamultiply')
end

function canvas.draw_buffer(index, ...)
    love.graphics.setBlendMode('alpha','premultiplied')
    if select(1, ...) then
        love.graphics.setColor(...)
    else
        love.graphics.setColor(1,1,1)
    end
    love.graphics.draw(_stored_canvases[index])
    love.graphics.setBlendMode('alpha','alphamultiply')
end

function canvas.get_info()
    local info = 'Index\t\tDimensions\n'
    for i, buffer in ipairs(_stored_canvases) do
        info = info .. i .. '\t\t' .. buffer:getWidth() .. ' x ' .. buffer:getHeight() .. '\n'
    end
    return info
end

return canvas