local check = {}

local _rules = {}

local _disabled_when_fused = true

local T = type

function check.disable_when_fused()
    _disabled_when_fused = true
end

function check.enable_when_fused()
    _disabled_when_fused = false
end

function _rules.integer_number(x) return math.floor(x) == x end
function _rules.positive_number(x) return x >= 0 end
function _rules.negative_number(x) return x <= 0 end
function _rules.normalized_number(x) return x >= 0 and x <= 1 end
function _rules.strictly_positive_number(x) return x > 0 end
function _rules.strictly_negative_number(x) return x < 0 end

local function check_argument(_, value, name, type, rules, range_min, range_max)
    if _disabled_when_fused and love.filesystem.isFused() then return end

    if type:sub(1,1) == '?' then 
        if value == nil then return end
        type = type:sub(2) 
    end

    local passes_test = T(value) == type

    if passes_test then
        if rules then
            for rule in rules:gmatch('([%a_]+)') do
                if _rules[rule..'_'..type] and not _rules[rule..'_'..type](value) then
                    passes_test = false
                    break
                end
            end
        end
        if range_min then
            passes_test = passes_test and value >= range_min
        end
        if range_max then
            passes_test = passes_test and value <= range_max
        end
    end

    if not passes_test then
        error(('argument %s must be a %s %s%s'):format(name, rules:gsub(' ', ', '):gsub('_', ' '), type, (range_min or range_max) and (' within the range [%s,%s]'):format(range_min or '', range_max or '') or ''), 3)
    end
end

setmetatable(check, {
    __call = check_argument,
})

return check