local pool = require(PATH..'.pool')
local input = require(PATH..'.input')

local event = {}
local event_tree_metatable = {}
event_tree_metatable.__index = event_tree_metatable

local traversal_pool = {}
local event_pool = {}

function event.new_event_tree()
    local t = {
        parent_node=nil,
        children={},
        event_handler=nil,
        unhandled_event_handler=nil,
        options = {},
    }
    setmetatable(t, event_tree_metatable)
    return t
end

function event.connect_tree(tree, parent_node)
    tree.parent_node = parent_node
    table.insert(parent_node.children, tree)
end

function event.disconnect_tree(tree)
    if tree.parent_node then
        local siblings=tree.parent_node.children

        for i=1, #siblings do
            if siblings[i] == tree then
                table.remove(siblings, i)
                break
            end
        end
        tree.parent_node = nil
    end
end

function event.tree_process_event(handler_tree, event, do_release_event)
    local traversal_table=pool.fetch(traversal_pool)
    local traversal_index=1
    local was_event_catched = false

    traversal_table[1] = handler_tree

    while traversal_index<=#traversal_table do
        local current_node=traversal_table[traversal_index]
        for i=1, #current_node.children do
            table.insert(traversal_table, current_node.children[i])
        end
        traversal_index=traversal_index+1
    end

    for i=#traversal_table, 1, -1 do
        local current_node=traversal_table[i]

        if current_node.event_handler then
            was_event_catched = current_node:event_handler(event) or was_event_catched
        end
        if current_node.unhandled_event_handler and not was_event_catched then
            was_event_catched = current_node:unhandled_event_handler(event)
        end
    end

    pool.release(traversal_pool, traversal_table)

    if do_release_event then
        pool.release(event_pool, event)
    end

    return was_event_catched
end

function event.new_event()
    return pool.fetch(event_pool)
end

function event.new_mousepressed_event(x, y, button)
    local e = event.new_event()
    e.name = "mousepressed"
    e.x = x
    e.y = y
    e.button = button
    return e
end

function event.new_mousereleased_event(x, y, button)
    local e = event.new_event()
    e.name = "mousereleased"
    e.x = x
    e.y = y
    e.button = button
    return e
end

function event.new_mousemoved_event(x, y, dx, dy)
    local e = event.new_event()
    e.name = "mousemoved"
    e.x = x
    e.y = y
    e.dx = dx
    e.dy = dy
    return e
end

function event.new_keypressed_event(key, scancode)
    local e = event.new_event()
    e.name = "keypressed"
    e.key = key
    e.scancode = scancode
    return e
end

function event.new_keyreleased_event(key, scancode)
    local e = event.new_event()
    e.name = "keyreleased"
    e.key = key
    e.scancode = scancode
    return e
end

function event.new_textinput_event(text)
    local e = event.new_event()
    e.name = "textinput"
    e.text = text
    return e
end

function event.new_update_event(delta_time)
    local e = event.new_event()
    e.name = "update"
    e.delta_time = delta_time
    return e
end

function event.release_event(e)
    pool.release(event_pool, e)
end

event_tree_metatable.connect = event.connect_tree
event_tree_metatable.disconnect = event.disconnect_tree
event_tree_metatable.process_event = event.tree_process_event

return event