local box = require(PATH..'.gui.box')
local event = require(PATH..'.gui.event')
local style = require(PATH..'.gui.def.style')
local new_textbox = require(PATH..'.gui.def.textbox')

local function new_number_picker(value, value_min, value_max)
    local picker = new_textbox(value)
    picker.value = value
    picker.value_min = value_min
    picker.value_max = value_max

    local incr_box = box.new_box()
    incr_box.width = 30
    incr_box.height = 15
    incr_box.draw_method = style.draw_button
    incr_box:append(picker.box)
    local decr_box = box.new_box()
    decr_box.width = 30
    decr_box.height = 15
    decr_box.draw_method = style.draw_button
    decr_box:append(picker.box)

    picker.box.width = 100
    picker.box.height = 30
    picker.box.layout_method = function()
        incr_box.global_transform:reset()
        incr_box.global_transform:apply(picker.box.global_transform)
        incr_box.global_transform:translate(picker.box.width - 30, 0)
        decr_box.global_transform:reset()
        decr_box.global_transform:apply(picker.box.global_transform)
        decr_box.global_transform:translate(picker.box.width - 30, 15)
    end

    picker.event_node.event_handler = function(e)
        picker.input_handler(e, picker.event_node.options)
        if e.name == 'mouse_pressed' then
            local value_changed = false
            if e.target_box == incr_box then
                incr_box.draw_options.is_pressed = true
                picker.value = picker.value + 1
                if picker.value_max then
                    picker.value = math.min(picker.value, picker.value_max)
                end
                value_changed = true
                event.catch_event(e)
            elseif e.target_box == decr_box then
                decr_box.draw_options.is_pressed = true
                picker.value = picker.value - 1
                if picker.value_min then
                    picker.value = math.max(picker.value, picker.value_min)
                end
                value_changed = true
                event.catch_event(e)
            end
            if value_changed then
                picker.input.text = tostring(picker.value)
                if picker.on_value_changed then
                    picker.on_value_changed(picker.value)
                end
            end
        elseif e.name == 'mouse_released' then
            incr_box.draw_options.is_pressed = false
            decr_box.draw_options.is_pressed = false
        end
    end

    return picker
end

--[[
local function new_input_number(value, value_min, value_max)
    local input = {
        box = box.new_box(),
        event_node = event.new_event_tree(),
        value = value or value_min or 0,
        value_min = value_min,
        value_max = value_max,
    }
    local incr_box = box.new_box()
    incr_box.width = 30
    incr_box.height = 15
    incr_box.draw_method = style.draw_button
    incr_box:append(input.box)
    local decr_box = box.new_box()
    decr_box.width = 30
    decr_box.height = 15
    decr_box.draw_method = style.draw_button
    decr_box:append(input.box)

    input.box.width = 100
    input.box.height = 30
    input.box.draw_method = style.draw_input
    input.box.draw_options.value = tostring(input.value)
    input.box.layout_method = function()
        incr_box.global_transform:reset()
        incr_box.global_transform:apply(input.box.global_transform)
        incr_box.global_transform:translate(input.box.width - 30, 0)
        decr_box.global_transform:reset()
        decr_box.global_transform:apply(input.box.global_transform)
        decr_box.global_transform:translate(input.box.width - 30, 15)
    end

    input.event_node.event_handler = function(e)
        if e.name == 'mouse_pressed' then
            local value_changed = false
            if e.target_box == incr_box then
                incr_box.draw_options.is_pressed = true
                input.value = input.value + 1
                if input.value_max then
                    input.value = math.min(input.value, input.value_max)
                end
                value_changed = true
                event.catch_event(e)
            elseif e.target_box == decr_box then
                decr_box.draw_options.is_pressed = true
                input.value = input.value - 1
                if input.value_min then
                    input.value = math.max(input.value, input.value_min)
                end
                value_changed = true
                event.catch_event(e)
            end
            if value_changed then
                input.box.draw_options.value = tostring(input.value)
                if input.on_value_changed then
                    input.on_value_changed(input.value)
                end
            end
        elseif e.name == 'mouse_released' then
            incr_box.draw_options.is_pressed = false
            decr_box.draw_options.is_pressed = false
        end
    end

    return input
end
--]]
return new_number_picker