local box = require(PATH..'.gui.box')
local layout = require(PATH..'.gui.layout')
local event = require(PATH..'.gui.event')
local style = require(PATH..'.gui.def.style')
local new_slider = require(PATH..'.gui.def.slider')
local new_num_picker = require(PATH..'.gui.def.num_picker')

local function new_rgb_picker(start_color)
    local picker = {
        color = start_color or {1,1,1},
        box = box.new_box(),
        event_node = event.new_event_tree(),
    }

    local color_box = box.new_box()
    color_box.width = 290
    color_box.height = 30
    color_box.draw_options.color = picker.color
    color_box.draw_method = style.draw_color

    local channels_box = box.new_box()
    channels_box.layout_method = layout.layout_grid
    channels_box.layout_options = {column_count=2, column_spacing = 10, row_spacing = 10}

    picker.box.layout_method = layout.layout_vertical
    picker.box.layout_options = {spacing = 15}

    picker.event_node.event_handler = function(name, channel_index, new_value)
        if name == "input_changed" then
            picker.color[channel_index] = new_value
            color_box.draw_options.color = picker.color
        end
    end

    for channel = 1, 3 do
        local slider = new_slider(1)
        slider.value_steps = 200
        slider.box.width = 200
        local num_input = new_num_picker(255,0,255)
        num_input.box.width = 80

        num_input.on_value_changed = function(v)
            slider.value = v / 255
            slider.box:layout()
            picker.event_node.event_handler("input_changed", channel, slider.value)
        end

        slider.on_value_changed = function(v)
            num_input.value = math.floor(255 * v)
            num_input.input.text = tostring(num_input.value)
            picker.event_node.event_handler("input_changed", channel, v)
        end

        slider.box:append(channels_box)
        num_input.box:append(channels_box)
        slider.event_node:connect(picker.event_node)
        num_input.event_node:connect(picker.event_node)
    end

    color_box:append(picker.box)
    channels_box:append(picker.box)
    channels_box:layout_tree()
    layout.adapt_to_content(channels_box)
    picker.box:layout_tree()
    layout.adapt_to_content(picker.box)

    return picker
end

return new_rgb_picker