local box = require(PATH..'.gui.box')
local event = require(PATH..'.gui.event')
local style = require(PATH..'.gui.def.style')

local function new_slider(value, value_steps)
    local slider = {
        box = box.new_box(),
        event_node = event.new_event_tree(),
        value = value or 0.5,
        value_steps = value_steps,
    }
    local is_user_changing = false
    local handle_box = box.new_box()
    handle_box.width = 20
    handle_box.height = 30
    handle_box.draw_method = style.draw_button
    handle_box:append(slider.box)

    slider.box.width = 150
    slider.box.height = 30
    slider.box.draw_method = style.draw_rail
    slider.box.layout_method = function()
        handle_box.global_transform:reset()
        handle_box.global_transform:apply(slider.box.global_transform)
        handle_box.global_transform:translate(slider.value * (slider.box.width - 20), 0)
    end

    slider.event_node.event_handler = function(e)
        if e.name == 'mouse_pressed' and e.target_box == handle_box then
            is_user_changing = true
            handle_box.draw_options.is_pressed = true
            event.catch_event(e)
        elseif is_user_changing and e.name == 'mouse_moved' then
            --local value_delta = e.dx / slider.box.width
            --slider.value = math.min(1, math.max(0, slider.value + value_delta))
            local mouse_input = select(1, slider.box.global_transform:inverseTransformPoint(e.x-10, 0)) / (slider.box.width-20)
            local value_before = slider.value
            local value_changed = false
            slider.value = math.min(1, math.max(0, mouse_input))

            if slider.value_steps then
                slider.value = math.floor(slider.value * slider.value_steps) / slider.value_steps
            end

            value_changed = slider.value ~= value_before

            if value_changed then
                slider.box:layout_tree()
                if slider.on_value_changed then
                    slider.on_value_changed(slider.value)
                end
            end

            event.catch_event(e)
        elseif e.name == 'mouse_released' then
            is_user_changing = false
            handle_box.draw_options.is_pressed = false
        end
    end

    return slider
end

return new_slider