local box = require(PATH..'.gui.box')
local event = require(PATH..'.gui.event')
local text_input = require(PATH..'.gui.text_input')
local font = require(PATH..'.font')
local style = require(PATH..'.gui.def.style')
local utf8 = require("utf8")

local function input_handler(self, e)
    o = self.options
    local tb = o.textbox
    local input = tb.input

    if tb.is_editing then
        if e.name == "mousemoved" then
            if tb.is_mouse_held then
                local local_x, local_y = tb.box.global_transform:inverseTransformPoint(e.x-3, e.y-3)
                local carriage_index = text_input.get_character_at_position(input, local_x, local_y)
                text_input.set_carriage_at(input, carriage_index)

                if tb.selection_first_character == carriage_index then
                    text_input.deselect(input)
                elseif tb.selection_first_character < carriage_index then
                    text_input.select_range(input, tb.selection_first_character+1, carriage_index)
                else
                    text_input.select_range(input, carriage_index+1, tb.selection_first_character)
                end
            end
        elseif e.name == "mousepressed" then
            if e.target == tb.box then
                local local_x, local_y = tb.box.global_transform:inverseTransformPoint(e.x-3, e.y-3)
                local carriage_index = text_input.get_character_at_position(input, local_x, local_y)
                text_input.set_carriage_at(input, carriage_index)
                tb.selection_first_character = carriage_index
                text_input.deselect(input)
                tb.is_mouse_held = true
            else
                tb.is_editing = false
                text_input.deselect(input)
                love.keyboard.setKeyRepeat(false)
            end
        elseif e.name == "mousereleased" then
            tb.is_mouse_held = false
        elseif e.name == "textinput" then
            text_input.insert_text(input, e.text)
        elseif e.name == "keypressed" then
            if e.key == "backspace" then
                if not text_input.has_selection(input) then
                    text_input.select_range(input, input.carriage_index, input.carriage_index)
                end
                text_input.insert_text(input, "")
            elseif e.key == "delete" then
                if not text_input.has_selection(input) then
                    text_input.select_range(input, input.carriage_index+1, input.carriage_index+1)
                end
                text_input.insert_text(input, "")
            end
        end
    else
        if e.name == "mousepressed" and e.target == tb.box then
            tb.is_editing = true
            text_input.set_carriage_at(input, #input.text)
            love.keyboard.setKeyRepeat(true)
        end
    end
end

local function new_textbox(text)
    local textbox = {
        box = box.new_box(),
        event_node = event.new_event_tree(),
        input = text_input.new_single_line(),
        is_editing = false,
        is_mouse_held = false,
        selection_first_character = nil,
        input_handler = input_handler,
    }

    textbox.input.text = tostring(text)
    textbox.box.width = 100
    textbox.box.height = 30
    textbox.box.draw_method = style.draw_textbox
    textbox.box.draw_options.textbox = textbox
    textbox.event_node.options.textbox = textbox

    textbox.event_node.unhandled_event_handler = input_handler

    return textbox
end

return new_textbox