local color = require(PATH..'.color')
local font = require(PATH..'.font')

local style = {}

local line_width = 2
local rounding = 4
local input_font_size = 16
local button_font_size = 16
local button_background = color.new_hex_color("ddd")
local button_shadow = color.new_hex_color("898")
local line_color = color.new_hex_color("444")
local rail_background = color.new_hex_color("788")
local input_text_color = color.new_hex_color("444")
local input_carriage_color = color.new_hex_color("002")
local input_selection_color = color.new_hex_color("8a9a")
local button_text_color = color.new_hex_color("444")

function style.draw_border(border_box)
    love.graphics.setColor(line_color)
    love.graphics.setLineWidth(line_width)
    love.graphics.rectangle("line",0,0,border_box.width,border_box.height,rounding)
end

function style.draw_rail(rail_box)
    love.graphics.setColor(rail_background)
    love.graphics.rectangle("fill",0,0,rail_box.width,rail_box.height,rounding)
    style.draw_border(rail_box)
end

function style.draw_button(button_box)
    local button = button_box.draw_options.widget

    love.graphics.setColor(button_shadow)
    love.graphics.rectangle("fill",0,0,button_box.width,button_box.height,rounding)
    love.graphics.setColor(button_background)
    love.graphics.rectangle("fill",0,0,button_box.width,button_box.height - (button.is_mouse_holding and 2 or 4),rounding)
    style.draw_border(button_box)
end

function style.draw_button_text(button_box)
    local button = button_box.draw_options.widget

    style.draw_button(button_box)
    love.graphics.setColor(button_text_color)
    love.graphics.printf(button.text, font.love(button_font_size), 0, 4+(button.is_mouse_holding and 2 or 0), button_box.width, "center")
end

function style.draw_color(color_box)
    love.graphics.setColor(color_box.draw_options.color)
    love.graphics.rectangle("fill",0,0,color_box.width,color_box.height,rounding)
    style.draw_border(color_box)
end

function style.draw_input(input_box)
    love.graphics.setColor(button_background)
    love.graphics.rectangle("fill",0,0,input_box.width,input_box.height,rounding)
    style.draw_border(input_box)
    love.graphics.setColor(input_text_color)
    love.graphics.print(input_box.draw_options.value, font.love(input_font_size), 3, 3)
end

function style.draw_textbox(main_box)
    local textbox = main_box.draw_options.textbox

    love.graphics.setColor(button_background)
    love.graphics.rectangle("fill",0,0,main_box.width,main_box.height,rounding)
    style.draw_border(main_box)

    if textbox.is_editing then
        local text_to_carriage = string.sub(textbox.input.text, 1, textbox.input.carriage_index)
        local carriage_x = textbox.input.font:getWidth(text_to_carriage)+3-1

        if textbox.input.selection_left_character then
            local text_to_selection = string.sub(textbox.input.text, 1, textbox.input.selection_left_character-1)
            local selected_text = string.sub(textbox.input.text, textbox.input.selection_left_character, textbox.input.selection_right_character)
            love.graphics.setColor(input_selection_color)
            love.graphics.rectangle("fill",3+textbox.input.font:getWidth(text_to_selection),3,textbox.input.font:getWidth(selected_text),textbox.input.font:getHeight(),2)
        end

        love.graphics.setColor(input_carriage_color)
        love.graphics.rectangle("fill",carriage_x,3,2,textbox.input.font:getHeight())
    end

    local sx,sy = main_box.global_transform:transformPoint(0,0)
    love.graphics.setScissor(sx,sy,main_box.width,main_box.height)
    love.graphics.setColor(input_text_color)
    love.graphics.print(textbox.input.text, textbox.input.font, 3, 3)
    love.graphics.setScissor()
end

function style.get_button_dimensions(text)
    return font.love(button_font_size):getWidth(text) + 2*rounding, font.love(button_font_size):getHeight() + 2*rounding
end

return style