local pool = require(PATH..'.pool')
local input = require(PATH..'.input')

local event = {}

local EventHandler = {}
EventHandler.__index = EventHandler

local traversal_pool = {}

local event_data_pool = {}

function event.new_event_handler()
    local t = {
        parent=nil,
        children={},
        handler_method=nil,
        unhandled_handler_method=nil,
        instance_data = {},
    }
    setmetatable(t, EventHandler)
    return t
end

function EventHandler:connect(parent)
    self:disconnect()

    self.parent = parent
    table.insert(parent_node.children, self)
end

function EventHandler:disconnect()
    if self.parent then
        local siblings=self.parent.children

        for i=1, #siblings do
            if siblings[i] == self then
                table.remove(siblings, i)
                break
            end
        end
        self.parent = nil
    end
end

function EventHandler:process_input()
    for name, a, b, c, d, e, f in input.poll() do
        local event_wrapper = event["new_"..name.."_data"]
        local event_data

        if event_wrapper then
            event_data = event_wrapper(a, b, c, d, e, f)
        else
            event_data = event.new_event_data(name, a, b, c, d, e, f)
        end

        self:process_event(event_data)

        event_data:release()
    end
end

function EventHandler:process_event(event)
    local traversal_table=pool.fetch(traversal_pool)
    local traversal_index=1
    local was_event_catched = false

    traversal_table[1] = self

    while traversal_index<=#traversal_table do
        local current_node=traversal_table[traversal_index]
        for i=1, #current_node.children do
            table.insert(traversal_table, current_node.children[i])
        end
        traversal_index=traversal_index+1
    end

    for i=#traversal_table, 1, -1 do
        local current_node=traversal_table[i]

        if current_node.handler_method then
            was_event_catched = current_node:handler_method(event) or was_event_catched
        end
        if current_node.unhandled_handler_method and not was_event_catched then
            was_event_catched = current_node:unhandled_handler_method(event)
        end
    end

    pool.release(traversal_pool, traversal_table)

    return was_event_catched
end

local Event = {}
Event.__index = Event

function event.new_event_data(name, a, b, c, d, e, f)
    local event_data = pool.fetch(event_data_pool)
    event_data.name = name
    event_data[1] = a
    event_data[2] = b
    event_data[3] = c
    event_data[4] = d
    event_data[5] = e
    event_data[6] = f
    setmetatable(event_data, Event)
    return event_data
end

function Event:release()
    pool.release(event_data_pool, self)
end

function Event:is(name)
    return self.name == name
end

function event.new_mousepressed_data(x, y, button)
    local e = event.new_event_data("mousepressed",x,y,button)
    e.x = x
    e.y = y
    e.button = button
    return e
end

function event.new_mousereleased_data(x, y, button)
    local e = event.new_event_data("mousereleased",x,y,button)
    e.x = x
    e.y = y
    e.button = button
    return e
end

function event.new_mousemoved_data(x, y, dx, dy)
    local e = event.new_event_data("mousemoved",x, y, dx, dy)
    e.x = x
    e.y = y
    e.dx = dx
    e.dy = dy
    return e
end

function event.new_wheelmoved_data(dx, dy)
    local e = event.new_event_data("wheelmoved", dx, dy)
    e.dx = dx
    e.dy = dy
    return e
end

function event.new_keypressed_data(key, scancode)
    local e = event.new_event_data("keypressed", key, scancode)
    e.key = key
    e.scancode = scancode
    return e
end

function event.new_keyreleased_data(key, scancode)
    local e = event.new_event_data("keyreleased", key, scancode)
    e.key = key
    e.scancode = scancode
    return e
end

function event.new_textinput_data(text)
    local e = event.new_event_data("textinput", text)
    e.text = text
    return e
end

return event