local layout = {}

function layout.layout_vertical(box)
    local x, y = 0,0
    local column_width=0
    local padding = box.layout_options.padding or 0
    local spacing = box.layout_options.spacing or 0
    local wrap = box.layout_options.wrap == true

    for i, child in ipairs(box.children) do
        if wrap and y> 0 and y+padding+child.height > box.height then
            y=0
            x=x+spacing+column_width
            column_width=0
        end

        child.layout_transform:reset()
        child.layout_transform:apply(box.global_transform)
        child.layout_transform:translate(x+padding, y+padding)
        child.layout_transform:translate(-box.scroll_x, -box.scroll_y)
        
        child:update_global_transform()

        box:update_visible_region(child)

        column_width=math.max(column_width, child.width)
        y=y+child.height+spacing
    end

    box.scroll_max_y=math.max(0, y-box.height+padding)
    box.scroll_max_x=math.max(0, x-box.width+column_width+2*padding)
end

function layout.layout_grid(box)
    local x, y = 0,0
    local column_count = box.layout_options.column_count or 1
    local padding = box.layout_options.padding or 0
    local padding_x = box.layout_options.padding_x or padding or 0
    local padding_y = box.layout_options.padding_y or padding or 0
    local column_spacing = box.layout_options.column_spacing or 0
    local row_spacing = box.layout_options.row_spacing or 0
    local column_widths={}
    local row_heights={}
    local column_offsets={}
    local row_offsets={}

    for i, child in ipairs(box.children) do
        local column_index=(i-1)%column_count +1
        local row_index=math.floor((i-1)/column_count)+1
        column_widths[column_index] = math.max(column_widths[column_index] or 0, child.width)
        row_heights[row_index] = math.max(row_heights[row_index] or 0, child.height)
    end

    for i, width in ipairs(column_widths) do
        if i==1 then
            column_offsets[i]=padding_x
        else
            column_offsets[i]=column_offsets[i-1] + column_widths[i-1] + column_spacing
        end
    end

    for i, height in ipairs(row_heights) do
        if i==1 then
            row_offsets[i]=padding_y
        else
            row_offsets[i]=row_offsets[i-1] + row_heights[i-1] + row_spacing
        end
    end

    for i, child in ipairs(box.children) do
        local column_index=(i-1)%column_count +1
        local row_index=math.floor((i-1)/column_count)+1
        local child_x, child_y

        child_x=column_offsets[column_index]
        child_y=row_offsets[row_index]

        child.layout_transform:reset()
        child.layout_transform:apply(box.global_transform)
        child.layout_transform:translate(child_x, child_y)
        child.layout_transform:translate(-box.scroll_x, -box.scroll_y)
        
        child:update_global_transform()

        box:update_visible_region(child)
    end

    if #box.children > 0 then
        box.scroll_max_y=math.max(0, row_offsets[#row_offsets]+row_heights[#row_offsets]-box.height+padding_y)
        --box.scroll_max_x=math.max(0, x-box.width+column_width+2*padding)
    end
end

function layout.adapt_to_content(box)
    local padding = box.layout_options.padding or 0
    local min_x, max_x
    local min_y, max_y
    local compare=false
    local box_x, box_y= box.global_transform:transformPoint(0,0)

    for i, child in ipairs(box.children) do
        local child_x, child_y

        child_x, child_y=child.global_transform:transformPoint(0,0)
        child_x=child_x-box_x
        child_y=child_y-box_y

        if compare then
            min_x=math.min(min_x,child_x)
            min_y=math.min(min_y,child_y)
            max_x=math.max(max_x,child_x+child.width)
            max_y=math.max(max_y,child_y+child.height)
        else
            min_x=child_x
            min_y=child_y
            max_x=child_x+child.width
            max_y=child_y+child.height
            compare=true
        end
    end

    box.width=max_x-min_x+2*padding
    box.height=max_y-min_y+2*padding
end

return layout