local oop = {}

local registered_classes = {}

function oop.new_class(super_class)
    local class_methods = {}
    class_methods.__index = class_methods
    local class_meta = {
        __call = function(_, ...)
            local instance = {}
            setmetatable(instance, class_methods)
            if class_methods.new then
                class_methods.new(instance, ...)
            end
            return instance
        end,
    }
    
    if super_class then
        super_class = registered_classes[super_class] or super_class
        --print(super_class, super_class.new)
        class_meta.__index = super_class
        class_methods.super = super_class
        class_methods.super_method = function(instance, method_name, ...) 
            return super_class[method_name](instance, ...) 
        end
    end

    setmetatable(class_methods, class_meta)
    
    return class_methods
end

function oop.new_registered_class(class_name, super_class)
    local class = oop.new_class(super_class)
    registered_classes[class_name] = class
    return class
end

function oop.new_instance_of(class_name, ...)
    return registered_classes[class_name](...)
end

return oop