local rectangle = {}

local floor, min, max = math.floor, math.min, math.max

function rectangle.width(left,top,right,bottom) return right-left end
function rectangle.height(left,top,right,bottom) return bottom-top end
function rectangle.size(left,top,right,bottom) return right-left, bottom-top end
function rectangle.center_x(left,top,right,bottom) return left + floor(0.5 * (right - left)) end
function rectangle.center_y(left,top,right,bottom) return top + floor(0.5 * (bottom - top)) end
function rectangle.center(left,top,right,bottom) return left + floor(0.5 * (right - left)), top + floor(0.5 * (bottom - top)) end
function rectangle.is_null(left,top,right,bottom) return right-left < 1 or bottom-top < 1 end
function rectangle.is_point_inside(point_x,point_y,left,top,right,bottom) 
    return (point_x >= left and point_x < right) and (point_y >= top and point_y < bottom)
end
function rectangle.minimum_bounding_rectangle(a_left,a_top,a_right,a_bottom,b_left,b_top,b_right,b_bottom)
    local mbr_left, mbr_top = min(a_left, b_left), min(a_top, b_top)
    local mbr_right, mbr_bottom = max(a_right, b_right), max(a_bottom, b_bottom)
    
    return mbr_left, mbr_top, mbr_right, mbr_bottom
end
function rectangle.are_intersecting(a_left,a_top,a_right,a_bottom,b_left,b_top,b_right,b_bottom)
    local mbr_left, mbr_top, mbr_right, mbr_bottom = rectangle.minimum_bounding_rectangle(a_left,a_top,a_right,a_bottom,b_left,b_top,b_right,b_bottom)
    local mbr_width, mbr_height = rectangle.size(mbr_left, mbr_top, mbr_right, mbr_bottom)
    local a_width, a_height = rectangle.size(a_left,a_top,a_right,a_bottom)
    local b_width, b_height = rectangle.size(b_left,b_top,b_right,b_bottom)

    return mbr_width < a_width + b_width and mbr_height < a_height + b_height
end
function rectangle.intersection(a_left,a_top,a_right,a_bottom,b_left,b_top,b_right,b_bottom)
    local mbr_left, mbr_top, mbr_right, mbr_bottom = rectangle.minimum_bounding_rectangle(a_left,a_top,a_right,a_bottom,b_left,b_top,b_right,b_bottom)
    local mbr_width, mbr_height = rectangle.size(mbr_left, mbr_top, mbr_right, mbr_bottom)
    local a_width, a_height = rectangle.size(a_left,a_top,a_right,a_bottom)
    local b_width, b_height = rectangle.size(b_left,b_top,b_right,b_bottom)

    local overlap_width = (a_width + b_width) - mbr_width
    local overlap_height = (a_height + b_height) - mbr_height
    local intersection_x, intersection_y = max(a_left, b_left), max(a_top, b_top)

    return intersection_x, intersection_y, intersection_x+overlap_width, intersection_y+overlap_height
end

function rectangle.copy(t, left,top,right,bottom) t[1],t[2],t[3],t[4] = left,top,right,bottom end
function rectangle.to_box(left,top,right,bottom) return left,top,right-left,bottom-top end

function rectangle.draw(mode,left,top,right,bottom)
    love.graphics.rectangle(mode, left,top, right-left, bottom-top)
end

return rectangle