local pool = require(PATH..".pool")

local schedule = {}

local event_handle_pool = {}
local event_pool = {}
local events = {}

function schedule.timer(seconds, callback)
    local event = {
        expiration_time = love.timer.getTime() + seconds,
        callback = callback,
        handle = pool.fetch(event_handle_pool),
        canceled = false,
    }
    events[event.handle] = event

    return event.handle
end

function schedule.update()
    for handle, event in pairs(events) do
        if event.canceled then
            pool.release(event_handle_pool, handle)
            pool.release(event_pool, event)
            events[handle] = nil
        elseif event.expiration_time <= love.timer.getTime() then
            if event.callback then
                event.callback()
            end
            event.canceled = true
        end
    end
end

function schedule.cancel(event_handle)
    local event = events[event_handle]
    if event then
        event.canceled = true
    end
end

function schedule.cancel_all()
    for handle, event in pairs(events) do
        event.canceled = true
    end
end

return schedule