local vector = {}
local floor = math.floor
local ceil = math.ceil

function vector.length(x, y)
    return math.sqrt(x*x + y*y)
end

function vector.normalized(x, y)
    if x == 0 and y == 0 then return 0, 0 end
    local di = 1 / vector.length(x, y)
    return x * di, y * di
end

function vector.translate(n, x, y)
    return x + n, y + n
end

function vector.add(x, y, w, z)
    return x + w, y + z
end

function vector.scale(n, x, y)
    return x * n, y * n
end

function vector.multiply(x, y, w, z)
    return x * w, y * z
end

function vector.dot(x, y, w, z)
    return x * w + y * z
end

function vector.rotate(angle, x, y)
    return math.cos(angle) * x - math.sin(angle) * y, math.sin(angle) * x + math.cos(angle) * y
end

function vector.distance(ax, ay, bx, by)
    return vector.length(ax - bx, ay - by)
end

function vector.reflect(x, y, normal_x, normal_y)
    return vector.add(x, y, vector.scale(-2 * vector.dot(normal_x, normal_y, x, y), normal_x, normal_y))
end

function vector.perpendicular(x, y)
    return y, -x
end

function vector.normal(x, y)
    return vector.normalized(vector.perpendicular(x, y))
end

function vector.angle_between(x, y, x2, y2)
    x2 = x2 or 1
    y2 = y2 or 0
    return math.acos(vector.dot(x, y, x2, y2) / (vector.length(x, y) * vector.length(x2, y2)))
end

function vector.slide(ax, ay, normal_x, normal_y)
    return vector.scale(vector.length(ax, ay), vector.perpendicular(normal_x, normal_y))
end

function vector.is_null(x, y)
    return x == 0 and y == 0
end

function vector.floor(x, y)
    return floor(x), floor(y)
end

function vector.ceil(x, y)
    return ceil(x), ceil(y)
end

function vector.round_p(x, y)
    return floor(x + 0.5), floor(y + 0.5)
end

function vector.round_n(x, y)
    return ceil(x - 0.5), ceil(y - 0.5)
end

return vector