DottedLine = require("libraries/classic"):extend()

function DottedLine:new(dot_diameter, spacing, phase_pixels)
    self.dot_diameter = dot_diameter or 4
    self.spacing = spacing or 2*self.dot_diameter
    self.phase_pixels = phase_pixels or 0
end

function DottedLine:draw_segment(x1, y1, x2, y2, phase_pixels)
    local length = gamekit.vector.distance(x1, y1, x2, y2)
    local pattern_length = self.dot_diameter + self.spacing
    local advancement = math.fmod(phase_pixels or 0, pattern_length)
    local norm_x, norm_y = gamekit.vector.normalized(x2 - x1, y2 - y1)

    love.graphics.setLineWidth(1)
    while advancement <= length do
        local dot_x, dot_y = x1 + norm_x * advancement, y1 + norm_y * advancement

        love.graphics.circle("fill", dot_x, dot_y, 0.5 * self.dot_diameter)
        love.graphics.circle("line", dot_x, dot_y, 0.5 * self.dot_diameter, 10)

        advancement = advancement + pattern_length
    end

    love.graphics.circle("fill", x2, y2, self.dot_diameter)
    love.graphics.circle("line", x2, y2, self.dot_diameter, 15)

    return advancement
end

function DottedLine:draw_polyline(...)
    local coords_count = select("#", ...)
    local phase_accumulator = self.phase_pixels

    for i = 1, coords_count - 2, 2 do
        phase_accumulator = self:draw_segment(
            select(i, ...), select(i + 1, ...), 
            select(i + 2, ...), select(i + 3, ...), 
            phase_accumulator)
    end
end