AttenuatorNode = Node:extend()

local text_font = love.graphics.newFont(24)
local half_text = love.graphics.newText(text_font, "1/2")
local three_fourths_text = love.graphics.newText(text_font, "3/4")
local one_fourth_text = love.graphics.newText(text_font, "1/4")

function AttenuatorNode:new(x, y, schema)
    Node.new(self, x, y, schema)
    self.mode = 1
end

function AttenuatorNode:get_max_inputs()
    return 1
end

function AttenuatorNode:get_output_color(ir, ig, ib)
    local coefficient = 0.25

    if self.mode == 1 then
        coefficient = 0.5
    elseif self.mode == 2 then
        coefficient = 0.75
    end

    return coefficient * ir, coefficient * ig, coefficient * ib
end

function AttenuatorNode:update()
    Node.update(self)

    if input:down("control") and input:pressed("pick") and self:is_point_inside(love.mouse.getPosition()) then
        self.mode = self.mode % 3 + 1
        self.schema:resolve()
    end
end

function AttenuatorNode:draw_body()
    Node.draw_body(self)

    local text = one_fourth_text

    if self.mode == 1 then
        text = half_text
    elseif self.mode == 2 then
        text = three_fourths_text
    end

    love.graphics.setColor(0,0,0)
    love.graphics.draw(text, self.x - math.floor(0.5 * text:getWidth()), self.y - math.floor(0.5 * text:getHeight()))
end

function AttenuatorNode:__tostring()
    return "AttenuatorNode"
end

