RefractorNode = Node:extend()

local text_font = love.graphics.newFont(30)
local r_text = love.graphics.newText(text_font, {{0,0,0},"R"})
local g_text = love.graphics.newText(text_font, {{0,0,0},"G"})
local b_text = love.graphics.newText(text_font, {{0,0,0},"B"})

function RefractorNode:new(x, y, schema)
    Node.new(self, x, y, schema)
    self.channel = 1
    self.radius = 0.9 * Node.Radius
end

function RefractorNode:get_input_y()
    return self.y - 0.9 * Node.Radius - 15
end

function RefractorNode:get_output_y()
    return self.y + 0.9 * Node.Radius + 15
end

function RefractorNode:update()
    Node.update(self)

    if input:down("control") and input:pressed("pick") and self:is_point_inside(love.mouse.getPosition()) then
        self.channel = self.channel % 3 + 1
        self.schema:resolve()
    end
end

function RefractorNode:get_max_inputs()
    return 1
end

function RefractorNode:get_output_color(ir, ig, ib)
    if self.channel == 1 then
        return ir, 0, 0
    elseif self.channel == 2 then
        return 0, ig, 0
    end
    return 0, 0, ib
end

function RefractorNode:draw_body()
    local drawn_text = b_text
    
    if self.channel == 1 then
        drawn_text = r_text
    elseif self.channel == 2 then
        drawn_text = g_text
    end

    love.graphics.setColor(1,1,1)
    love.graphics.rectangle("fill", self.x - self.radius, self.y - self.radius, 2*self.radius, 2*self.radius, 5)
    love.graphics.setLineWidth(3)
    love.graphics.setColor(0,0,0)
    love.graphics.rectangle("line", self.x - self.radius, self.y - self.radius, 2*self.radius, 2*self.radius, 5)

    love.graphics.setColor(1,1,1)
    love.graphics.draw(drawn_text, self.x - 0.5 * drawn_text:getWidth(), self.y - 0.5 * drawn_text:getHeight())
end

function RefractorNode:__tostring()
    return "RefractorNode"
end

